/**
 * Copyright (c) 2009 Mark Logic Corporation. All rights reserved.
 */
package com.marklogic.cpox;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Map;

/**
 * @author Michael Blakeley, Mark Logic Corporation
 * 
 */
public class NullWorker extends Worker {

    /**
     * @param controller
     * @param index
     */
    public NullWorker(Controller controller, int index) {
        super(controller, index);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.marklogic.cpox.Worker#bookmarkAction(java.lang.String)
     */
    @Override
    protected InputStream bookmarkAction(String user, String id)
            throws ActionException {
        logger.info("bookmark " + user + ", " + id);
        if (bookmarkedIds.contains(id)) {
            throw new ActionException(user + "already bookmarked " + id);
        }
        // this technique, like everything in this class, is cheating.
        // TODO: can we keep other implementors from cheating like this?
        return new ByteArrayInputStream(("" + (1 + bookmarkedIds.size()))
                .getBytes());
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.marklogic.cpox.Worker#searchAction(java.lang.String[],
     * java.lang.String, int, java.util.Map)
     */
    @SuppressWarnings("unused")
    @Override
    protected InputStream searchAction(String[] query, String language,
            int start, Map<String, String> facets) throws ActionException {
        logger.info("search " + query.length + ", " + start + ", "
                + facets.size());
        String xml = "<response total=\"987654210\" start=\"" + start
                + "\">\n";
        int base = Math.abs(random.nextInt());
        for (int i = 0; i < 10; i++) {
            xml += "<result index=\"" + i + "\" uri=\"entry" + (base++)
                    + "\"/>\n";
        }
        for (int i = 0; i < 4; i++) {
            xml += "<facet name=\"facet" + i + "\">";
            for (int j = 5; j > 0; j--) {
                xml += "<facet-value name=\"value" + j + "\" count=\""
                        + j + "\">value" + j + "</facet-value>";
            }
            xml += "</facet>";
        }
        xml += "</response>\n";
        return new ByteArrayInputStream(xml.getBytes());
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.marklogic.cpox.Worker#updateAction(java.lang.String,
     * java.lang.String)
     */
    @SuppressWarnings("unused")
    @Override
    protected InputStream updateAction(String id, String token)
            throws ActionException {
        logger.info("update " + id + ", " + token);
        return new ByteArrayInputStream(new byte[0]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.marklogic.cpox.Worker#verifyAction(java.lang.String)
     */
    @SuppressWarnings("unused")
    @Override
    protected InputStream verifyAction(String token)
            throws ActionException {
        logger.info("verify " + token);
        // this technique, like everything in this class, is cheating.
        // TODO: can we keep other implementors from cheating like this?
        return new ByteArrayInputStream(("" + updatedIds.size())
                .getBytes());
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.marklogic.cpox.Worker#viewAction(java.lang.String)
     */
    @SuppressWarnings("unused")
    @Override
    protected InputStream viewAction(String id) throws ActionException {
        logger.info("view " + id);
        return new ByteArrayInputStream(new byte[0]);
    }

}
