/**
 * Copyright (c) 2009 Mark Logic Corporation. All rights reserved.
 */
package com.marklogic.cpox;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.nio.charset.CharsetDecoder;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.regex.Pattern;

/**
 * @author Michael Blakeley, Mark Logic Corporation
 * 
 */
public class Utilities {

    protected static final int BUFFER_SIZE = 16 * 1024;

    public static DateFormat ISO8601 = new SimpleDateFormat(
            "yyyy-MM-dd'T'HH:mm:ssZ");

    public static String formatDateTime(Date date) {
        if (date == null) {
            return formatDateTime(new Date());
        }

        // format in (almost) ISO8601 format
        String dateStr = ISO8601.format(date);

        // re-map the time zone from 0000 to 00:00
        return dateStr.substring(0, 22) + ":" + dateStr.substring(22);
    }

    /**
     * @return
     */
    public static String formatDateTime() {
        return formatDateTime(new Date());
    }

    protected static Pattern[] patterns = new Pattern[] {
            Pattern.compile("&"), Pattern.compile("<"),
            Pattern.compile(">") };

    public static String escapeXml(String _in) {
        if (null == _in)
            return "";
        return patterns[2].matcher(
                patterns[1].matcher(
                        patterns[0].matcher(_in).replaceAll("&amp;"))
                        .replaceAll("&lt;")).replaceAll("&gt;");
    }

    public static String join(Collection<String> thisPath, String _delim) {
        return join(thisPath.toArray(new String[0]), _delim);
    }

    /**
     * @param _items
     * @param _delim
     * @return
     */
    public static String join(String[] _items, String _delim) {
        StringBuilder rval = new StringBuilder();
        for (int i = 0; i < _items.length; i++) {
            if (i == 0) {
                rval.append(_items[0]);
            } else {
                rval.append(_delim).append(_items[i]);
            }
        }
        return rval.toString();
    }

    /**
     * @param _items
     * @param _delim
     * @return
     */
    public static String join(Object[] _items, String _delim) {
        StringBuilder rval = new StringBuilder();
        for (int i = 0; i < _items.length; i++) {
            if (i == 0) {
                rval.append(_items[0]);
            } else {
                rval.append(_delim).append(_items[i]);
            }
        }
        return rval.toString();
    }

    /**
     * @param sb
     * @throws IOException
     */
    public static void read(Reader input, StringBuilder sb)
            throws IOException {
        // uses a reader, so charset translation should be ok
        int size;
        char[] buf = new char[BUFFER_SIZE];
        while ((size = input.read(buf)) > -1) {
            sb.append(buf, 0, size);
        }
        // mark for gc
        buf = null;
    }

    /**
     * @param _is
     * @param _decoder
     * @return
     * @throws IOException
     */
    public static String read(InputStream _is, CharsetDecoder _decoder)
            throws IOException {
        StringBuilder sb = new StringBuilder();
        read(new InputStreamReader(_is, _decoder), sb);
        return sb.toString();
    }

    /**
     * @param _is
     * @param _decoder
     * @return
     * @throws IOException
     */
    public static byte[] read(InputStream _is) throws IOException {
        if (null == _is) {
            throw new IOException("null InputStream");
        }
        ByteArrayOutputStream os = new ByteArrayOutputStream(_is
                .available());
        byte[] buf = new byte[BUFFER_SIZE];
        int len = 0;
        while ((len = _is.read(buf)) != -1) {
            os.write(buf, 0, len);
        }
        // mark for gc
        buf = null;
        os.flush();
        return os.toByteArray();
    }

}
